@@ -1,4 +1,4 @@
-/*	$NetBSD: eval.c,v 1.42 1998/02/05 08:34:49 christos Exp $	*/
+/*	$NetBSD: eval.c,v 1.46 1999/06/26 16:31:47 christos Exp $	*/
 
 /*-
  * Copyright (c) 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)eval.c	8.9 (Berkeley) 6/8/95";
 #else
-__RCSID("$NetBSD: eval.c,v 1.42 1998/02/05 08:34:49 christos Exp $");
+__RCSID("$NetBSD: eval.c,v 1.46 1999/06/26 16:31:47 christos Exp $");
 #endif
 #endif /* not lint */
 
@@ -435,6 +435,7 @@ expredir(n)
 		fn.lastp = &fn.list;
 		oexitstatus = exitstatus;
 		switch (redir->type) {
+		case NFROMTO:
 		case NFROM:
 		case NTO:
 		case NAPPEND:
@@ -545,10 +546,19 @@ evalbackcmd(n, result)
 		exitstatus = 0;
 		goto out;
 	}
+#ifdef notyet
+	/*
+	 * For now we disable executing builtins in the same
+	 * context as the shell, because we are not keeping
+	 * enough state to recover from changes that are
+	 * supposed only to affect subshells. eg. echo "`cd /`"
+	 */
 	if (n->type == NCMD) {
 		exitstatus = oexitstatus;
 		evalcommand(n, EV_BACKCMD, result);
-	} else {
+	} else
+#endif
+	{
 		exitstatus = 0;
 		if (pipe(pip) < 0)
 			error("Pipe call failed");
@@ -820,9 +830,8 @@ cmddone:
 		cmdenviron = NULL;
 		if (e != EXSHELLPROC) {
 			commandname = savecmdname;
-			if (flags & EV_EXIT) {
+			if (flags & EV_EXIT)
 				exitshell(exitstatus);
-			}
 		}
 		handler = savehandler;
 		if (e != -1) {
@@ -854,7 +863,6 @@ cmddone:
 			setvareq(sp->text, VEXPORT|VSTACK);
 		envp = environment();
 		shellexec(argv, envp, pathval(), cmdentry.u.index);
-		/*NOTREACHED*/
 	}
 	goto out;
 
@@ -1007,7 +1015,6 @@ execcmd(argc, argv)
 		for (sp = cmdenviron; sp ; sp = sp->next)
 			setvareq(sp->text, VEXPORT|VSTACK);
 		shellexec(argv + 1, environment(), pathval(), 0);
-
 	}
 	return 0;
 }
