@@ -1,4 +1,4 @@
-/*	$NetBSD: expand.c,v 1.37 1998/03/10 19:11:07 christos Exp $	*/
+/*	$NetBSD: expand.c,v 1.47 1999/04/30 17:54:17 he Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)expand.c	8.5 (Berkeley) 5/15/95";
 #else
-__RCSID("$NetBSD: expand.c,v 1.37 1998/03/10 19:11:07 christos Exp $");
+__RCSID("$NetBSD: expand.c,v 1.47 1999/04/30 17:54:17 he Exp $");
 #endif
 #endif /* not lint */
 
@@ -53,6 +53,7 @@ __RCSID("$NetBSD: expand.c,v 1.37 1998/03/10 19:11:07 christos Exp $");
 #include <unistd.h>
 #include <pwd.h>
 #include <stdlib.h>
+#include <stdio.h>
 
 /*
  * Routines to expand arguments to commands.  We have to deal with
@@ -74,7 +75,6 @@ __RCSID("$NetBSD: expand.c,v 1.37 1998/03/10 19:11:07 christos Exp $");
 #include "memalloc.h"
 #include "error.h"
 #include "mystring.h"
-#include "arith.h"
 #include "show.h"
 
 /*
@@ -96,14 +96,15 @@ struct ifsregion ifsfirst;	/* first struct in list of ifs regions */
 struct ifsregion *ifslastp;	/* last struct in list */
 struct arglist exparg;		/* holds expanded arg list */
 
-STATIC int argstr __P((char *, int));
+STATIC void argstr __P((char *, int));
 STATIC char *exptilde __P((char *, int));
 STATIC void expbackq __P((union node *, int, int));
-STATIC int subevalvar __P((char *, char *, int, int, int, int, int *));
+STATIC int subevalvar __P((char *, char *, int, int, int, int));
 STATIC char *evalvar __P((char *, int));
 STATIC int varisset __P((char *, int));
 STATIC void varvalue __P((char *, int, int));
 STATIC void recordregion __P((int, int, int));
+STATIC void removerecordregions __P((int)); 
 STATIC void ifsbreakup __P((char *, struct arglist *));
 STATIC void ifsfree __P((void));
 STATIC void expandmeta __P((struct strlist *, int));
@@ -111,7 +112,7 @@ STATIC void expmeta __P((char *, char *));
 STATIC void addfname __P((char *));
 STATIC struct strlist *expsort __P((struct strlist *));
 STATIC struct strlist *msort __P((struct strlist *, int));
-STATIC int pmatch __P((char *, char *));
+STATIC int pmatch __P((char *, char *, int));
 STATIC char *cvtnum __P((int, char *));
 
 /*
@@ -149,7 +150,7 @@ expandarg(arg, arglist, flag)
 	STARTSTACKSTR(expdest);
 	ifsfirst.next = NULL;
 	ifslastp = NULL;
-	(void)argstr(arg->narg.text, flag);
+	argstr(arg->narg.text, flag);
 	if (arglist == NULL) {
 		return;			/* here document expanded */
 	}
@@ -188,7 +189,7 @@ expandarg(arg, arglist, flag)
  * $@ like $* since no splitting will be performed.
  */
 
-STATIC int
+STATIC void
 argstr(p, flag)
 	char *p;
 	int flag;
@@ -196,7 +197,6 @@ argstr(p, flag)
 	char c;
 	int quotes = flag & (EXP_FULL | EXP_CASE);	/* do CTLESC */
 	int firsteq = 1;
-	int recorded = 0;
 
 	if (*p == '~' && (flag & (EXP_TILDE | EXP_VARTILDE)))
 		p = exptilde(p, flag);
@@ -205,6 +205,13 @@ argstr(p, flag)
 		case '\0':
 		case CTLENDVAR: /* ??? */
 			goto breakloop;
+		case CTLQUOTEMARK:
+			/* "$@" syntax adherence hack */
+			if (p[0] == CTLVAR && p[2] == '@' && p[3] == '=')
+				break;
+			if ((flag & EXP_FULL) != 0)
+				STPUTC(c, expdest);
+			break;
 		case CTLESC:
 			if (quotes)
 				STPUTC(c, expdest);
@@ -216,7 +223,6 @@ argstr(p, flag)
 			break;
 		case CTLBACKQ:
 		case CTLBACKQ|CTLQUOTE:
-			recorded |= (c & CTLQUOTE) == 0;
 			expbackq(argbackq->n, c & CTLQUOTE, flag);
 			argbackq = argbackq->next;
 			break;
@@ -245,7 +251,7 @@ argstr(p, flag)
 		}
 	}
 breakloop:;
-	return recorded;
+	return;
 }
 
 STATIC char *
@@ -255,13 +261,15 @@ exptilde(p, flag)
 {
 	char c, *startp = p;
 	struct passwd *pw;
-	char *home;
+	const char *home;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
 
 	while ((c = *p) != '\0') {
 		switch(c) {
 		case CTLESC:
 			return (startp);
+		case CTLQUOTEMARK:
+			return (startp);
 		case ':':
 			if (flag & EXP_VARTILDE)
 				goto done;
@@ -285,7 +293,7 @@ done:
 		goto lose;
 	*p = c;
 	while ((c = *home++) != '\0') {
-		if (quotes && SQSYNTAX[c] == CCTL)
+		if (quotes && SQSYNTAX[(int)c] == CCTL)
 			STPUTC(CTLESC, expdest);
 		STPUTC(c, expdest);
 	}
@@ -296,6 +304,47 @@ lose:
 }
 
 
+STATIC void 
+removerecordregions(endoff)
+	int endoff;
+{
+	if (ifslastp == NULL)
+		return;
+
+	if (ifsfirst.endoff > endoff) {
+		while (ifsfirst.next != NULL) {
+			struct ifsregion *ifsp;
+			INTOFF;
+			ifsp = ifsfirst.next->next;
+			ckfree(ifsfirst.next);
+			ifsfirst.next = ifsp;
+			INTON;
+		}
+		if (ifsfirst.begoff > endoff)
+			ifslastp = NULL;
+		else {
+			ifslastp = &ifsfirst;
+			ifsfirst.endoff = endoff;
+		}
+		return;
+	}
+	
+	ifslastp = &ifsfirst;
+	while (ifslastp->next && ifslastp->next->begoff < endoff)
+		ifslastp=ifslastp->next;
+	while (ifslastp->next != NULL) {
+		struct ifsregion *ifsp;
+		INTOFF;
+		ifsp = ifslastp->next->next;
+		ckfree(ifslastp->next);
+		ifslastp->next = ifsp;
+		INTON;
+	}
+	if (ifslastp->endoff > endoff)
+		ifslastp->endoff = endoff;
+}
+
+
 /*
  * Expand arithmetic expression.  Backup to start of expression,
  * evaluate, place result in (backed up) result, adjust string position.
@@ -306,9 +355,11 @@ expari(flag)
 {
 	char *p, *start;
 	int result;
+	int begoff;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
+	int quoted;
 
-	ifsfree();
+	/*	ifsfree(); */
 
 	/*
 	 * This routine is slightly over-complicated for
@@ -323,7 +374,7 @@ expari(flag)
 	CHECKSTRSPACE(8, expdest);
 	USTPUTC('\0', expdest);
 	start = stackblock();
-	p = expdest;
+	p = expdest - 1;
 	while (*p != CTLARI && p >= start)
 		--p;
 	if (*p != CTLARI)
@@ -332,14 +383,23 @@ expari(flag)
 		for (p = start; *p != CTLARI; p++)
 			if (*p == CTLESC)
 				p++;
+
+	if (p[1] == '"')
+		quoted=1;
+	else
+		quoted=0;
+	begoff = p - start;
+	removerecordregions(begoff);
 	if (quotes)
-		rmescapes(p+1);
-	result = arith(p+1);
+		rmescapes(p+2);
+	result = arith(p+2);
 	fmtstr(p, 10, "%d", result);
 
 	while (*p++)
 		;
 
+	if (quoted == 0)
+		recordregion(begoff, p - 1 - start, 0);
 	result = expdest - p + 1;
 	STADJUST(-result, expdest);
 }
@@ -397,7 +457,7 @@ expbackq(cmd, quoted, flag)
 		}
 		lastc = *p++;
 		if (lastc != '\0') {
-			if (quotes && syntax[lastc] == CCTL)
+			if (quotes && syntax[(int)lastc] == CCTL)
 				STPUTC(CTLESC, dest);
 			STPUTC(lastc, dest);
 		}
@@ -426,24 +486,24 @@ expbackq(cmd, quoted, flag)
 
 
 STATIC int
-subevalvar(p, str, strloc, subtype, startloc, varflags, recorded)
+subevalvar(p, str, strloc, subtype, startloc, varflags)
 	char *p;
 	char *str;
 	int strloc;
 	int subtype;
 	int startloc;
 	int varflags;
-	int *recorded;
 {
 	char *startp;
 	char *loc = NULL;
+	char *q;
 	int c = 0;
 	int saveherefd = herefd;
 	struct nodelist *saveargbackq = argbackq;
 	int amount;
 
 	herefd = -1;
-	*recorded = argstr(p, 0);
+	argstr(p, 0);
 	STACKSTRNUL(expdest);
 	herefd = saveherefd;
 	argbackq = saveargbackq;
@@ -469,49 +529,61 @@ subevalvar(p, str, strloc, subtype, startloc, varflags, recorded)
 		error("%.*s: parameter %snot set", p - str - 1,
 		      str, (varflags & VSNUL) ? "null or "
 					      : nullstr);
-		return 0;
+		/* NOTREACHED */
 
 	case VSTRIMLEFT:
 		for (loc = startp; loc < str; loc++) {
 			c = *loc;
 			*loc = '\0';
-			if (patmatch(str, startp)) {
-				*loc = c;
+			if (patmatch(str, startp, varflags & VSQUOTE))
 				goto recordleft;
-			}
 			*loc = c;
+			if ((varflags & VSQUOTE) && *loc == CTLESC)
+			        loc++;
 		}
 		return 0;
 
 	case VSTRIMLEFTMAX:
-		for (loc = str - 1; loc >= startp; loc--) {
+		for (loc = str - 1; loc >= startp;) {
 			c = *loc;
 			*loc = '\0';
-			if (patmatch(str, startp)) {
-				*loc = c;
+			if (patmatch(str, startp, varflags & VSQUOTE))
 				goto recordleft;
-			}
 			*loc = c;
+			loc--;
+			if ((varflags & VSQUOTE) && loc > startp &&
+			    *(loc - 1) == CTLESC) {
+				for (q = startp; q < loc; q++)
+					if (*q == CTLESC)
+						q++;
+				if (q > loc)
+					loc--;
+			}
 		}
 		return 0;
 
 	case VSTRIMRIGHT:
-		for (loc = str - 1; loc >= startp; loc--) {
-			if (patmatch(str, loc)) {
-				amount = loc - expdest;
-				STADJUST(amount, expdest);
-				return 1;
+	        for (loc = str - 1; loc >= startp;) {
+			if (patmatch(str, loc, varflags & VSQUOTE))
+				goto recordright;
+			loc--;
+			if ((varflags & VSQUOTE) && loc > startp &&
+			    *(loc - 1) == CTLESC) { 
+				for (q = startp; q < loc; q++)
+					if (*q == CTLESC)
+						q++;
+				if (q > loc)
+					loc--;
 			}
 		}
 		return 0;
 
 	case VSTRIMRIGHTMAX:
 		for (loc = startp; loc < str - 1; loc++) {
-			if (patmatch(str, loc)) {
-				amount = loc - expdest - 1;
-				STADJUST(amount, expdest);
-				return 1;
-			}
+			if (patmatch(str, loc, varflags & VSQUOTE))
+				goto recordright;
+			if ((varflags & VSQUOTE) && *loc == CTLESC)
+			        loc++;
 		}
 		return 0;
 
@@ -520,11 +592,19 @@ subevalvar(p, str, strloc, subtype, startloc, varflags, recorded)
 	}
 
 recordleft:
+	*loc = c;
 	amount = ((str - 1) - (loc - startp)) - expdest;
 	STADJUST(amount, expdest);
 	while (loc != str - 1)
 		*startp++ = *loc++;
 	return 1;
+
+recordright:
+	amount = loc - expdest;
+	STADJUST(amount, expdest);
+	STPUTC('\0', expdest);
+	STADJUST(-1, expdest);
+	return 1;
 }
 
 
@@ -542,14 +622,13 @@ evalvar(p, flag)
 	int varflags;
 	char *var;
 	char *val;
-	char *pat;
+	int patloc;
 	int c;
 	int set;
 	int special;
 	int startloc;
 	int varlen;
 	int easy;
-	int recorded;
 	int quotes = flag & (EXP_FULL | EXP_CASE);
 
 	varflags = *p++;
@@ -572,10 +651,7 @@ again: /* jump here after setting a variable with ${var=text} */
 			set = 1;
 	}
 	varlen = 0;
-	if (flag & EXP_RECORD)
-		startloc = 0;
-	else
-		startloc = expdest - stackblock();
+	startloc = expdest - stackblock();
 	if (set && subtype != VSPLUS) {
 		/* insert the value of the variable */
 		if (special) {
@@ -594,7 +670,7 @@ again: /* jump here after setting a variable with ${var=text} */
 			}
 			else {
 				while (*val) {
-					if (quotes && syntax[*val] == CCTL)
+					if (quotes && syntax[(int)*val] == CCTL)
 						STPUTC(CTLESC, expdest);
 					STPUTC(*val++, expdest);
 				}
@@ -626,7 +702,7 @@ record:
 	case VSPLUS:
 	case VSMINUS:
 		if (!set) {
-			(void)argstr(p, flag);
+		        argstr(p, flag);
 			break;
 		}
 		if (easy)
@@ -644,27 +720,32 @@ record:
 		 * right after it
 		 */
 		STPUTC('\0', expdest);
-		pat = expdest;
-		if (subevalvar(p, NULL, expdest - stackblock(), subtype,
-			       startloc, varflags, &recorded))
-			goto record;
-		else {
-			int amount = (expdest - pat) + 1;
+		patloc = expdest - stackblock();
+		if (subevalvar(p, NULL, patloc, subtype,
+			       startloc, varflags) == 0) {
+			int amount = (expdest - stackblock() - patloc) + 1;
 			STADJUST(-amount, expdest);
 		}
-		break;
+		/* Remove any recorded regions beyond start of variable */
+		removerecordregions(startloc);
+		goto record;
 
 	case VSASSIGN:
 	case VSQUESTION:
 		if (!set) {
-			if (subevalvar(p, var, 0, subtype, startloc, varflags,
-			    &recorded)) {
+			if (subevalvar(p, var, 0, subtype, startloc,
+				       varflags)) {
 				varflags &= ~VSNUL;
+				/* 
+				 * Remove any recorded regions beyond 
+				 * start of variable 
+				 */
+				removerecordregions(startloc);
 				goto again;
 			}
 			break;
 		}
-		if (easy && !recorded)
+		if (easy)
 			goto record;
 		break;
 
@@ -760,7 +841,7 @@ varvalue(name, quoted, allow_split)
 	if (allow_split) { \
 		syntax = quoted? DQSYNTAX : BASESYNTAX; \
 		while (*p) { \
-			if (syntax[*p] == CCTL) \
+			if (syntax[(int)*p] == CCTL) \
 				STPUTC(CTLESC, expdest); \
 			STPUTC(*p++, expdest); \
 		} \
@@ -792,19 +873,23 @@ numvar:
 		}
 		break;
 	case '@':
-		if (allow_split) {
-			sep = '\0';
-			goto allargs;
+		if (allow_split && quoted) {
+			for (ap = shellparam.p ; (p = *ap++) != NULL ; ) {
+				STRTODEST(p);
+				if (*ap)
+					STPUTC('\0', expdest);
+			}
+			break;
 		}
 		/* fall through */
 	case '*':
-		sep = ((p=bltinlookup("IFS", 1)) == NULL || p[0] == '\0')
-			? ' ' : p[0];
-
-allargs:
+		if (ifsset() != 0)
+			sep = ifsval()[0];
+		else
+			sep = ' ';
 		for (ap = shellparam.p ; (p = *ap++) != NULL ; ) {
 			STRTODEST(p);
-			if (*ap)
+			if (*ap && sep)
 				STPUTC(sep, expdest);
 		}
 		break;
@@ -871,45 +956,69 @@ ifsbreakup(string, arglist)
 	char *q;
 	char *ifs;
 	int ifsspc;
+	int nulonly;
 
 
 	start = string;
+	ifsspc = 0;
+	nulonly = 0;
 	if (ifslastp != NULL) {
 		ifsp = &ifsfirst;
 		do {
 			p = string + ifsp->begoff;
-			ifs = ifsp->nulonly? nullstr : ifsval();
-			ifsspc = strchr(ifs, ' ') != NULL;
+			nulonly = ifsp->nulonly;
+			ifs = nulonly ? nullstr : 
+				( ifsset() ? ifsval() : " \t\n" );
+			ifsspc = 0;
 			while (p < string + ifsp->endoff) {
 				q = p;
 				if (*p == CTLESC)
 					p++;
-				if (strchr(ifs, *p++)) {
-					if (q > start || !ifsspc) {
-						*q = '\0';
-						sp = (struct strlist *)stalloc(sizeof *sp);
-						sp->text = start;
-						*arglist->lastp = sp;
-						arglist->lastp = &sp->next;
+				if (strchr(ifs, *p)) {
+					if (!nulonly)
+						ifsspc = (strchr(" \t\n", *p) != NULL);
+					/* Ignore IFS whitespace at start */
+					if (q == start && ifsspc) {
+						p++;
+						start = p;
+						continue;
 					}
-					if (ifsspc) {
+					*q = '\0';
+					sp = (struct strlist *)stalloc(sizeof *sp);
+					sp->text = start;
+					*arglist->lastp = sp;
+					arglist->lastp = &sp->next;
+					p++;
+					if (!nulonly) {
 						for (;;) {
-							if (p >= string + ifsp->endoff)
+							if (p >= string + ifsp->endoff) {
 								break;
+							}
 							q = p;
 							if (*p == CTLESC)
 								p++;
-							if (strchr(ifs, *p++) == NULL) {
+							if (strchr(ifs, *p) == NULL ) {
 								p = q;
 								break;
-							}
+							} else if (strchr(" \t\n",*p) == NULL) {
+								if (ifsspc) {
+									p++;
+									ifsspc = 0;
+								} else {
+									p = q;
+									break;
+								}
+							} else
+								p++;
 						}
 					}
 					start = p;
-				}
+				} else
+					p++;
 			}
 		} while ((ifsp = ifsp->next) != NULL);
-		if (*start || (!ifsspc && start > string)) {
+		if (*start || (!ifsspc && start > string && 
+			(nulonly || 1))) {
 			sp = (struct strlist *)stalloc(sizeof *sp);
 			sp->text = start;
 			*arglist->lastp = sp;
@@ -1030,6 +1139,8 @@ expmeta(enddir, name)
 			if (*q == '!')
 				q++;
 			for (;;) {
+				while (*q == CTLQUOTEMARK)
+					q++;
 				if (*q == CTLESC)
 					q++;
 				if (*q == '/' || *q == '\0')
@@ -1043,6 +1154,8 @@ expmeta(enddir, name)
 			metaflag = 1;
 		} else if (*p == '\0')
 			break;
+		else if (*p == CTLQUOTEMARK)
+			continue;
 		else if (*p == CTLESC)
 			p++;
 		if (*p == '/') {
@@ -1055,6 +1168,8 @@ expmeta(enddir, name)
 		if (enddir != expdir)
 			metaflag++;
 		for (p = name ; ; p++) {
+			if (*p == CTLQUOTEMARK)
+				continue;
 			if (*p == CTLESC)
 				p++;
 			*enddir++ = *p;
@@ -1069,6 +1184,8 @@ expmeta(enddir, name)
 	if (start != name) {
 		p = name;
 		while (p < start) {
+			while (*p == CTLQUOTEMARK)
+				p++;
 			if (*p == CTLESC)
 				p++;
 			*enddir++ = *p++;
@@ -1093,12 +1210,17 @@ expmeta(enddir, name)
 		*endname++ = '\0';
 	}
 	matchdot = 0;
-	if (start[0] == '.' || (start[0] == CTLESC && start[1] == '.'))
+	p = start;
+	while (*p == CTLQUOTEMARK)
+		p++;
+	if (*p == CTLESC)
+		p++;
+	if (*p == '.')
 		matchdot++;
 	while (! int_pending() && (dp = readdir(dirp)) != NULL) {
 		if (dp->d_name[0] == '.' && ! matchdot)
 			continue;
-		if (patmatch(start, dp->d_name)) {
+		if (patmatch(start, dp->d_name, 0)) {
 			if (atend) {
 				scopy(dp->d_name, enddir);
 				addfname(expdir);
@@ -1207,23 +1329,25 @@ msort(list, len)
  */
 
 int
-patmatch(pattern, string)
+patmatch(pattern, string, squoted)
 	char *pattern;
 	char *string;
+	int squoted;	/* string might have quote chars */
 	{
 #ifdef notdef
 	if (pattern[0] == '!' && pattern[1] == '!')
 		return 1 - pmatch(pattern + 2, string);
 	else
 #endif
-		return pmatch(pattern, string);
+		return pmatch(pattern, string, squoted);
 }
 
 
 STATIC int
-pmatch(pattern, string)
+pmatch(pattern, string, squoted)
 	char *pattern;
 	char *string;
+	int squoted;
 	{
 	char *p, *q;
 	char c;
@@ -1235,25 +1359,41 @@ pmatch(pattern, string)
 		case '\0':
 			goto breakloop;
 		case CTLESC:
+			if (squoted && *q == CTLESC)
+				q++;
 			if (*q++ != *p++)
 				return 0;
 			break;
+		case CTLQUOTEMARK:
+			continue;
 		case '?':
+			if (squoted && *q == CTLESC)
+				q++;
 			if (*q++ == '\0')
 				return 0;
 			break;
 		case '*':
 			c = *p;
-			if (c != CTLESC && c != '?' && c != '*' && c != '[') {
+			while (c == CTLQUOTEMARK || c == '*')
+				c = *++p;
+			if (c != CTLESC &&  c != CTLQUOTEMARK &&
+			    c != '?' && c != '*' && c != '[') {
 				while (*q != c) {
+					if (squoted && *q == CTLESC &&
+					    q[1] == c)
+						break;
 					if (*q == '\0')
 						return 0;
+					if (squoted && *q == CTLESC)
+						q++;
 					q++;
 				}
 			}
 			do {
-				if (pmatch(p, q))
+				if (pmatch(p, q, squoted))
 					return 1;
+				if (squoted && *q == CTLESC)
+					q++;
 			} while (*q++ != '\0');
 			return 0;
 		case '[': {
@@ -1265,6 +1405,8 @@ pmatch(pattern, string)
 			if (*endp == '!')
 				endp++;
 			for (;;) {
+				while (*endp == CTLQUOTEMARK)
+					endp++;
 				if (*endp == '\0')
 					goto dft;		/* no matching ] */
 				if (*endp == CTLESC)
@@ -1279,14 +1421,20 @@ pmatch(pattern, string)
 			}
 			found = 0;
 			chr = *q++;
+			if (squoted && chr == CTLESC)
+				chr = *q++;
 			if (chr == '\0')
 				return 0;
 			c = *p++;
 			do {
+				if (c == CTLQUOTEMARK)
+					continue;
 				if (c == CTLESC)
 					c = *p++;
 				if (*p == '-' && p[1] != ']') {
 					p++;
+					while (*p == CTLQUOTEMARK)
+						p++;
 					if (*p == CTLESC)
 						p++;
 					if (chr >= c && chr <= *p)
@@ -1302,6 +1450,8 @@ pmatch(pattern, string)
 			break;
 		}
 dft:	        default:
+			if (squoted && *q == CTLESC)
+				q++;
 			if (*q++ != c)
 				return 0;
 			break;
@@ -1322,16 +1472,20 @@ breakloop:
 void
 rmescapes(str)
 	char *str;
-	{
+{
 	char *p, *q;
 
 	p = str;
-	while (*p != CTLESC) {
+	while (*p != CTLESC && *p != CTLQUOTEMARK) {
 		if (*p++ == '\0')
 			return;
 	}
 	q = p;
 	while (*p) {
+		if (*p == CTLQUOTEMARK) {
+			p++;
+			continue;
+		}
 		if (*p == CTLESC)
 			p++;
 		*q++ = *p++;
@@ -1358,10 +1512,10 @@ casematch(pattern, val)
 	argbackq = pattern->narg.backquote;
 	STARTSTACKSTR(expdest);
 	ifslastp = NULL;
-	(void)argstr(pattern->narg.text, EXP_TILDE | EXP_CASE);
+	argstr(pattern->narg.text, EXP_TILDE | EXP_CASE);
 	STPUTC('\0', expdest);
 	p = grabstackstr(expdest);
-	result = patmatch(p, val);
+	result = patmatch(p, val, 0);
 	popstackmark(&smark);
 	return result;
 }
