@@ -1,4 +1,4 @@
-/*	$NetBSD: jobs.c,v 1.25 1998/02/04 23:49:25 thorpej Exp $	*/
+/*	$NetBSD: jobs.c,v 1.30 1999/04/05 14:59:35 mycroft Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)jobs.c	8.5 (Berkeley) 5/4/95";
 #else
-__RCSID("$NetBSD: jobs.c,v 1.25 1998/02/04 23:49:25 thorpej Exp $");
+__RCSID("$NetBSD: jobs.c,v 1.30 1999/04/05 14:59:35 mycroft Exp $");
 #endif
 #endif /* not lint */
 
@@ -50,6 +50,7 @@ __RCSID("$NetBSD: jobs.c,v 1.25 1998/02/04 23:49:25 thorpej Exp $");
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <paths.h>
 #include <sys/types.h>
 #include <sys/param.h>
 #ifdef BSD
@@ -297,9 +298,11 @@ showjobs(change)
 		procno = jp->nprocs;
 		for (ps = jp->ps ; ; ps++) {	/* for each process */
 			if (ps == jp->ps)
-				fmtstr(s, 64, "[%d] %d ", jobno, ps->pid);
+				fmtstr(s, 64, "[%d] %ld ", jobno, 
+				    (long)ps->pid);
 			else
-				fmtstr(s, 64, "    %d ", ps->pid);
+				fmtstr(s, 64, "    %ld ", 
+				    (long)ps->pid);
 			out1str(s);
 			col = strlen(s);
 			s[0] = '\0';
@@ -426,7 +429,7 @@ jobidcmd(argc, argv)
 
 	jp = getjob(argv[1]);
 	for (i = 0 ; i < jp->nprocs ; ) {
-		out1fmt("%d", jp->ps[i].pid);
+		out1fmt("%ld", (long)jp->ps[i].pid);
 		out1c(++i < jp->nprocs? ' ' : '\n');
 	}
 	return 0;
@@ -488,8 +491,7 @@ currentjob:
 		}
 	}
 	error("No such job: %s", name);
-	/*NOTREACHED*/
-	return NULL;
+	/* NOTREACHED */
 }
 
 
@@ -572,6 +574,8 @@ forkshell(jp, n, mode)
 {
 	int pid;
 	int pgrp;
+	const char *devnull = _PATH_DEVNULL;
+	/* const */ char *nullerr = "Can't open %s";
 
 	TRACE(("forkshell(%%%d, 0x%lx, %d) called\n", jp - jobtab, (long)n,
 	    mode));
@@ -622,8 +626,8 @@ forkshell(jp, n, mode)
 			if ((jp == NULL || jp->nprocs == 0) &&
 			    ! fd0_redirected_p ()) {
 				close(0);
-				if (open("/dev/null", O_RDONLY) != 0)
-					error("Can't open /dev/null");
+				if (open(devnull, O_RDONLY) != 0)
+					error(nullerr, devnull);
 			}
 		}
 #else
@@ -633,8 +637,8 @@ forkshell(jp, n, mode)
 			if ((jp == NULL || jp->nprocs == 0) &&
 			    ! fd0_redirected_p ()) {
 				close(0);
-				if (open("/dev/null", O_RDONLY) != 0)
-					error("Can't open /dev/null");
+				if (open(devnull, O_RDONLY) != 0)
+					error(nullerr, devnull);
 			}
 		}
 #endif
@@ -728,9 +732,17 @@ waitforjob(jp)
 		st = WTERMSIG(status) + 128;
 	if (! JOBS || jp->state == JOBDONE)
 		freejob(jp);
+#if 0
+	/*
+	 * XXXX
+	 * Why was this here?  It causes us to lose SIGINTs unless the current
+	 * command happens to also catch the SIGINT and exit with the right
+	 * status.  I don't see how that can possibly be correct.  -- mycroft
+	 */
 	CLEAR_PENDING_INT;
 	if (WIFSIGNALED(status) && WTERMSIG(status) == SIGINT)
-		kill(getpid(), SIGINT);
+		raise(SIGINT);
+#endif
 	INTON;
 	return st;
 }
@@ -1057,6 +1069,8 @@ until:
 		p = "<";  i = 0;  goto redir;
 	case NFROMFD:
 		p = "<&";  i = 0;  goto redir;
+	case NFROMTO:
+		p = "<>";  i = 0;  goto redir;
 redir:
 		if (n->nfile.fd != i) {
 			s[0] = n->nfile.fd + '0';
