@@ -1,4 +1,4 @@
-/*	$NetBSD: options.c,v 1.25 1997/07/04 21:02:16 christos Exp $	*/
+/*	$NetBSD: options.c,v 1.28 1998/07/28 11:41:57 mycroft Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)options.c	8.2 (Berkeley) 5/4/95";
 #else
-__RCSID("$NetBSD: options.c,v 1.25 1997/07/04 21:02:16 christos Exp $");
+__RCSID("$NetBSD: options.c,v 1.28 1998/07/28 11:41:57 mycroft Exp $");
 #endif
 #endif /* not lint */
 
@@ -241,6 +241,7 @@ setoption(flag, val)
 			return;
 		}
 	error("Illegal option -%c", flag);
+	/* NOTREACHED */
 }
 
 
@@ -441,7 +442,7 @@ atend:
 				err |= setvarsafe("OPTARG", s, 0);
 			}
 			else {
-				out1fmt("Illegal option -%c\n", c);
+				outfmt(&errout, "Illegal option -%c\n", c);
 				(void) unsetvar("OPTARG");
 			}
 			c = '?';
@@ -460,7 +461,7 @@ atend:
 				c = ':';
 			}
 			else {
-				out1fmt("No arg for -%c option\n", c);
+				outfmt(&errout, "No arg for -%c option\n", c);
 				(void) unsetvar("OPTARG");
 				c = '?';
 			}
