@@ -1,4 +1,4 @@
-/*	$NetBSD: parser.c,v 1.37 1997/07/15 17:45:53 christos Exp $	*/
+/*	$NetBSD: parser.c,v 1.42 1999/02/04 16:17:39 christos Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)parser.c	8.7 (Berkeley) 5/16/95";
 #else
-__RCSID("$NetBSD: parser.c,v 1.37 1997/07/15 17:45:53 christos Exp $");
+__RCSID("$NetBSD: parser.c,v 1.42 1999/02/04 16:17:39 christos Exp $");
 #endif
 #endif /* not lint */
 
@@ -122,8 +122,8 @@ STATIC int readtoken __P((void));
 STATIC int xxreadtoken __P((void));
 STATIC int readtoken1 __P((int, char const *, char *, int));
 STATIC int noexpand __P((char *));
-STATIC void synexpect __P((int));
-STATIC void synerror __P((char *));
+STATIC void synexpect __P((int)) __attribute__((noreturn));
+STATIC void synerror __P((char *)) __attribute__((noreturn));
 STATIC void setprompt __P((int));
 
 
@@ -481,6 +481,7 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 		goto checkneg;
 	default:
 		synexpect(-1);
+		/* NOTREACHED */
 	}
 
 	/* Now check for redirection which may follow command */
@@ -959,27 +960,36 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 						USTPUTC('\\', out);
 					if (SQSYNTAX[c] == CCTL)
 						USTPUTC(CTLESC, out);
+					else if (eofmark == NULL)
+						USTPUTC(CTLQUOTEMARK, out);
 					USTPUTC(c, out);
 					quotef++;
 				}
 				break;
 			case CSQUOTE:
+				if (eofmark == NULL)
+					USTPUTC(CTLQUOTEMARK, out);
 				syntax = SQSYNTAX;
 				break;
 			case CDQUOTE:
+				if (eofmark == NULL)
+					USTPUTC(CTLQUOTEMARK, out);
 				syntax = DQSYNTAX;
 				dblquote = 1;
 				break;
 			case CENDQUOTE:
-				if (eofmark) {
+				if (eofmark != NULL && arinest == 0 &&
+				    varnest == 0) {
 					USTPUTC(c, out);
 				} else {
-					if (arinest)
+					if (arinest) {
 						syntax = ARISYNTAX;
-					else
+						dblquote = 0;
+					} else if (eofmark == NULL) {
 						syntax = BASESYNTAX;
+						dblquote = 0;
+					}
 					quotef++;
-					dblquote = 0;
 				}
 				break;
 			case CVAR:	/* '$' */
@@ -1006,6 +1016,10 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 						if (--arinest == 0) {
 							USTPUTC(CTLENDARI, out);
 							syntax = prevsyntax;
+							if (syntax == DQSYNTAX)
+								dblquote = 1;
+							else
+								dblquote = 0;
 						} else
 							USTPUTC(')', out);
 					} else {
@@ -1119,8 +1133,8 @@ parseredir: {
 		}
 	} else {	/* c == '<' */
 		np->nfile.fd = 0;
-		c = pgetc();
-		if (c == '<') {
+		switch (c = pgetc()) {
+		case '<':
 			if (sizeof (struct nfile) != sizeof (struct nhere)) {
 				np = (union node *)stalloc(sizeof (struct nhere));
 				np->nfile.fd = 0;
@@ -1134,11 +1148,20 @@ parseredir: {
 				heredoc->striptabs = 0;
 				pungetc();
 			}
-		} else if (c == '&')
+			break;
+
+		case '&':
 			np->type = NFROMFD;
-		else {
+			break;
+
+		case '>':
+			np->type = NFROMTO;
+			break;
+
+		default:
 			np->type = NFROM;
 			pungetc();
+			break;
 		}
 	}
 	if (fd != '\0')
@@ -1345,8 +1368,7 @@ done:
                 STPUTC('\0', out);
                 savelen = out - stackblock();
                 if (savelen > 0) {
-                        str = ckmalloc(savelen);
-                        memcpy(str, stackblock(), savelen);
+			str = grabstackstr(out);
 			setinputstring(str, 1);
                 }
         }
@@ -1412,6 +1434,10 @@ parsearith: {
 		prevsyntax = syntax;
 		syntax = ARISYNTAX;
 		USTPUTC(CTLARI, out);
+		if (dblquote)
+			USTPUTC('"',out);
+		else
+			USTPUTC(' ',out);
 	} else {
 		/*
 		 * we collapse embedded arithmetic expansion to
@@ -1447,9 +1473,11 @@ noexpand(text)
 
 	p = text;
 	while ((c = *p++) != '\0') {
+		if (c == CTLQUOTEMARK)
+			continue;
 		if (c == CTLESC)
 			p++;
-		else if (BASESYNTAX[c] == CCTL)
+		else if (BASESYNTAX[(int)c] == CCTL)
 			return 0;
 	}
 	return 1;
@@ -1497,6 +1525,7 @@ synexpect(token)
 		fmtstr(msg, 64, "%s unexpected", tokname[lasttoken]);
 	}
 	synerror(msg);
+	/* NOTREACHED */
 }
 
 
@@ -1508,6 +1537,7 @@ synerror(msg)
 		outfmt(&errout, "%s: %d: ", commandname, startlinno);
 	outfmt(&errout, "Syntax error: %s\n", msg);
 	error((char *)NULL);
+	/* NOTREACHED */
 }
 
 STATIC void
