@@ -1,4 +1,4 @@
-/*	$NetBSD: trap.c,v 1.17 1997/07/04 21:02:24 christos Exp $	*/
+/*	$NetBSD: trap.c,v 1.21 1999/03/27 13:46:19 christos Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)trap.c	8.5 (Berkeley) 6/5/95";
 #else
-__RCSID("$NetBSD: trap.c,v 1.17 1997/07/04 21:02:24 christos Exp $");
+__RCSID("$NetBSD: trap.c,v 1.21 1999/03/27 13:46:19 christos Exp $");
 #endif
 #endif /* not lint */
 
@@ -174,7 +174,7 @@ setsignal(signo)
 	if (rootshell && action == S_DFL) {
 		switch (signo) {
 		case SIGINT:
-			if (iflag)
+			if (iflag || minusc || sflag == 0)
 				action = S_CATCH;
 			break;
 		case SIGQUIT:
@@ -232,6 +232,7 @@ setsignal(signo)
 		case S_IGN:	sigact = SIG_IGN;	break;
 	}
 	*t = action;
+	siginterrupt(signo, 1);
 	return (long)signal(signo, sigact);
 }
 
@@ -380,4 +381,5 @@ l1:   handler = &loc2;			/* probably unnecessary */
 	setjobctl(0);
 #endif
 l2:   _exit(status);
+	/* NOTREACHED */
 }
